/*
** This file is part of the ViTE project.
**
** This software is governed by the CeCILL-A license under French law
** and abiding by the rules of distribution of free software. You can
** use, modify and/or redistribute the software under the terms of the
** CeCILL-A license as circulated by CEA, CNRS and INRIA at the following
** URL: "http://www.cecill.info".
** 
** As a counterpart to the access to the source code and rights to copy,
** modify and redistribute granted by the license, users are provided
** only with a limited warranty and the software's author, the holder of
** the economic rights, and the successive licensors have only limited
** liability.
** 
** In this respect, the user's attention is drawn to the risks associated
** with loading, using, modifying and/or developing or reproducing the
** software by the user in light of its specific status of free software,
** that may mean that it is complicated to manipulate, and that also
** therefore means that it is reserved for developers and experienced
** professionals having in-depth computer knowledge. Users are therefore
** encouraged to load and test the software's suitability as regards
** their requirements in conditions enabling the security of their
** systems and/or data to be ensured and, more generally, to use and
** operate it in the same conditions as regards security.
** 
** The fact that you are presently reading this means that you have had
** knowledge of the CeCILL-A license and that you accept its terms.
**
**
** ViTE developpers are (for version 0.* to 1.0):
**
**        - COULOMB Kevin
**        - FAVERGE Mathieu
**        - JAZEIX Johnny
**        - LAGRASSE Olivier
**        - MARCOUEILLE Jule
**        - NOISETTE Pascal
**        - REDONDY Arthur
**        - VUCHENER Clément 
**
*/
/*!
 *\file Render_opengl.hpp
 */

#ifndef RENDER_OPENGL_HPP
#define RENDER_OPENGL_HPP

class Render_opengl;
class Interface_console;

#include "render.hpp"
#include "Render_area.hpp"


/*!
 * \brief This class redefined the OpenGL widget - QGLWidget - to display the trace.
 */
class Render_opengl : /*public QGLWidget,*/ public Render_area, public Render
{
    Q_OBJECT
    
protected:
    
    /*!
     * \brief Contains the parent instance.
     */
    Interface_console* _core;

    /***********************************
     *
     * The wait screen drawing.
     *
     **********************************/
     
    /***********************************
     * The wait list Attributes.
     **********************************/

    /*!
     * \brief The container GLu list.
     */
    GLuint _list_containers;
    
    /*!
     * \brief The state GLu list.
     */
    GLuint _list_states;
    
     /*!
      * \brief The counter GLu list.
      */
    GLuint _list_counters;

    
    /*!
     * \brief The wait GLu list.
     */
    //  GLuint _wait_list;
    


public:

    /***********************************
     *
     * Constructor and destructor.
     *
     **********************************/

    /*!
     * \brief The default constructor
     */
    Render_opengl(Render_opengl* render, Interface_console* core, QWidget *parent);

    /*!
     * \brief The destructor
     */
    virtual ~Render_opengl();

    /*!
     * \brief Set Statistics and Informations about input trace
     */
    void set_total_width(Element_pos){}

    /*!
     * \brief Set Statistics and Informations about input trace
     */
    void set_total_time(Times){}

    /*!
     * \brief display the scale 
     */
    void display_time_scale(){}


    /***********************************
     *
     * Default QGLWidget functions.
     *
     **********************************/
     
    /*!
     * \brief Call by the system to initialize the OpenGL render area.
     */
      void initializeGL();
     
    /*!
     * \brief Call by the system when the render area was resized (occurs during a window resizement).
     * \param width : the new width of the render area.
     * \param height : the new height of the render area. 
     */
    void resizeGL(int width, int height);
     
    /*!
     * \brief Call by the system each time the render area need to be updated.
     */
    void paintGL();



    /***********************************
     *
     * Building functions.
     *
     **********************************/

    /*!
     * \brief This function constructs the trace.
     */
    virtual bool display_build() =0;

    /*!
     * \brief This function releases the trace.
     */
    virtual bool display_unbuild() =0;

    /*!
     * \brief Proceeds with the initialization of the OpenGL draw functions.
     */
    virtual void start_draw() =0;

    /*!
     * \brief Creates and opens the display list for container draws.
     */
    virtual void start_draw_containers() =0;

    /*!
     * \brief Draw a container according to the parameters
     * \param x the x position of the container
     * \param y the y position of the container
     * \param w the width of the container
     * \param h the height of the container
     */
    virtual void draw_container(const Element_pos x, const Element_pos y, const Element_pos w, const Element_pos h) =0;

    /*!
     * \brief Draw the text of a container.
     * \param x the x position of the text.
     * \param y the y position of the text.
     * \param value the string value of the text.
     *
     * This function stores text in a list. This list will be display each time the render area need to be updated.
     */
    virtual void draw_container_text(const Element_pos x, const Element_pos y, const std::string value) =0;

    /*!
     * \brief Closes the container display list.
     */
    virtual void end_draw_containers() =0;
     
    /*!
     * \brief Creates and opens the display list for stater draws.
     */
    virtual void start_draw_states() =0;

    /*!
     * \brief Draw a state of the trace.
     * \param start the beginning time of the state.
     * \param end the ending time of the state.
     * \param base vertical position of the state.
     * \param height the state height.
     * \param r the red color rate of the state.
     * \param g the green color rate of the state.
     * \param b the blue color rate of the state.
     */
    virtual void draw_state(const Element_pos start , const Element_pos end, const Element_pos base, const Element_pos height, const Element_col r, const Element_col g, const Element_col b) =0;

    /*!
     * \brief Closes the state display list.
     */
    virtual void end_draw_states() =0;

    /*!
     * \brief Open the arrow display list.
     */
    virtual void start_draw_arrows() =0;

    /*!
     * \brief Draw an arrow.
     * \param start_time the beginning time of the arrow.
     * \param end_time the ending time of the arrow.
     * \param start_height vertical position of the begining time of the arrow.
     * \param end_height vertical position of the ending time of the arrow.
     *
     * This function stores all the information of the arrow to display it each time the render area need to be updated.
     */
    virtual void draw_arrow(const Element_pos start_time, const Element_pos end_time, const Element_pos start_height, const Element_pos end_height) =0;

    /*!
     * \brief Closes the arrow display list.
     */
    virtual void end_draw_arrows() =0;

    /*!
     * \brief Draw arrows contained in the Arrow_ vector
     * \param arrows An arrow vector.
     */
    virtual void draw_stored_arrows(std::vector<Arrow_> &arrows) =0;

    /*!
     * \brief Draw an event.
     * \param time time when the event occurs.
     * \param height vertical position of the event.
     * \param container_height information to draw event. It corresponds to the container height when they are drawn horizontally.
     *
     * This function stores all the information of the event to display it each time the render area need to be updated.
     */
    virtual void draw_event(const Element_pos time, const Element_pos height,  const Element_pos container_height) =0;

    /*!
     * \brief Draw events contained in the Event_ vector
     * \param events An event vector.
     */
    virtual void draw_stored_events(std::vector<Event_> &events) =0;

    /*!
     * \brief Creates and opens the display list for counter draws.
     */
    virtual void start_draw_counter() =0;
     
    /*!
     * \brief Draw a point of the counter.
     * \param x x position of the point.
     * \param y y position of the point.
     *
     * Each time counter is increased, this function is called with the coordinates of the new point.
     */
    virtual void draw_counter(const Element_pos x, const Element_pos y) =0;
     
    /*!
     * \brief Closes the counter display list.
     */
    virtual void end_draw_counter() =0;

    /*!
     * \brief Do nothing (it is present for compatibility of the Render class).
     */
    virtual void end_draw() =0;

    

    /***********************************
     *
     * Displaying functions.
     *
     **********************************/

    /*!
     * \brief Display on screen containers between container_begin and container_end.
     * \param container_begin integer value : id of the first container.
     * \param container_end integer value : id of the last container.
     */
    virtual void display_container(Element_count container_begin, Element_count container_end) =0;

    /*!
     * \brief Display on screen states between timer_begin and time_end, 
     * container_begin and container_end and with timer width between depth_begin and depth_end.
     * \param time_begin floating point value : time of the first state.
     * \param time_end floating point value : time of the last state.
     * \param container_begin integer value : id of the first container.
     * \param container_end integer value : id of the last container.
     * \param depth_begin floating point value : width of the narrowest state.
     * \param depth_end floating point value : width of the widest state.
     */
    virtual void display_state(Element_pos time_begin, Element_pos time_end, 
                       Element_count container_begin, Element_count container_end, 
                       Element_pos depth_begin, Element_pos depth_end) =0;

    /*!
     * \brief Display on screen arrows between timer_begin and time_end, 
     * container_begin and container_end and with timer width between depth_begin and depth_end.
     * \param time_begin floating point value : time of the smallest arrow time value.
     * \param time_end floating point value : time of the higher arrow time value.
     * \param container_begin integer value : id of the first container.
     * \param container_end integer value : id of the last container.
     * \param depth_begin floating point value : the narrowest difference between 
     * the beginning time and the ending time of the arrow.
     * \param depth_end floating point value : width of the widest difference between
     * the beginning time and the ending time of the arrow.
     */
    virtual void display_arrow(Element_pos time_begin, Element_pos time_end, 
                       Element_count container_begin, Element_count container_end, 
                       Element_pos depth_begin, Element_pos depth_end) =0;

    /*!
     * \brief Display on screen events between timer_begin and time_end, 
     * container_begin and container_end.
     * \param time_begin floating point value : time of the first event.
     * \param time_end floating point value : time of the last event.
     * \param container_begin integer value : id of the first container.
     * \param container_end integer value : id of the last container.
     */
    virtual void display_event(Element_pos time_begin, Element_pos time_end, 
                       Element_count container_begin, Element_count container_end) =0;  

    /*!
     * \brief Display on screen counters between timer_begin and time_end, 
     * container_begin and container_end.
     * \param time_begin floating point value : time of the smallest counter time value.
     * \param time_end floating point value : time of the higher counter time value.
     * \param container_begin integer value : id of the first container.
     * \param container_end integer value : id of the last container.
     */
    virtual void display_counter(Element_pos time_begin, Element_pos time_end, 
                         Element_count container_begin, Element_count container_end) =0;   
    
};



#endif
